from django.forms import Select


class CustomSelect(Select):
    """
    Temporarily fixes an issue with wagtail whereby select dropdowns are not getting selected correctly due to a
    missing wrapping div
    """
    template_name = "headless_waf_builder/widgets/custom_select.html"



from wagtail import blocks

from .condition_block import ConditionBlock
from ...forms.widgets.custom_select import CustomSelect
import headless_waf_builder.constants as consts


class BlankConditionBlock(ConditionBlock):

    rule = blocks.ChoiceBlock(
        choices=consts.FIELD_RULE_CHOICES_BLANK,
        default=consts.FIELD_RULE_IS_BLANK,
        widget=CustomSelect,
    )

    value = None

    class Meta:
        icon = 'cog'



# Generated by Django 4.2.20 on 2025-05-01 22:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('headless_waf_builder', '0004_adjusting_thanks_page_content_richtext_features'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailformpage',
            name='use_google_recaptcha',
            field=models.BooleanField(default=False, help_text='Please tick this option to enable Google Recaptcha'),
        ),
        migrations.AddField(
            model_name='formpage',
            name='use_google_recaptcha',
            field=models.BooleanField(default=False, help_text='Please tick this option to enable Google Recaptcha'),
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-23 04:15

from django.db import migrations
import wagtail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('headless_waf_builder', '0003_switch_streamfield_to_use_json_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailformpage',
            name='form',
            field=wagtail.fields.StreamField([('singleline', 14), ('email', 15), ('html', 18), ('dropdown', 22), ('radio', 24), ('checkboxes', 27), ('checkbox', 29), ('multiselect', 30), ('multiline', 31), ('url', 32), ('hidden', 34), ('number', 35)], block_lookup={0: ('wagtail.blocks.CharBlock', (), {'help_text': 'The label of the form field.', 'max_length': 255, 'required': True}), 1: ('wagtail.blocks.BooleanBlock', (), {'default': False, 'help_text': 'Tick this box to make this a required field.', 'required': False}), 2: ('wagtail.blocks.IntegerBlock', (), {'help_text': 'Set a maximum length for this field. e.g. 100', 'required': False}), 3: ('wagtail.blocks.CharBlock', (), {'help_text': 'Set a default value for this field.', 'max_length': 255, 'required': False}), 4: ('wagtail.blocks.CharBlock', (), {'help_text': 'Text to assist the user in populating this field.', 'max_length': 255, 'required': False}), 5: ('wagtail.blocks.ChoiceBlock', [], {'choices': [('show', 'Show'), ('hide', 'Hide')], 'help_text': 'What conditional action would you like to perform on this field?'}), 6: ('wagtail.blocks.CharBlock', (), {'help_text': 'The name of a field in the form', 'max_length': 255, 'required': True}), 7: ('wagtail.blocks.ChoiceBlock', [], {'choices': [('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')]}), 8: ('wagtail.blocks.CharBlock', (), {'help_text': '', 'max_length': 255, 'required': True}), 9: ('wagtail.blocks.StructBlock', [[('field_name', 6), ('rule', 7), ('value', 8)]], {}), 10: ('wagtail.blocks.ChoiceBlock', [], {'choices': [('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]}), 11: ('wagtail.blocks.StructBlock', [[('field_name', 6), ('rule', 10)]], {}), 12: ('wagtail.blocks.StreamBlock', [[('condition', 9), ('blank_condition', 11)]], {'required': False}), 13: ('wagtail.blocks.StructBlock', [[('action', 5), ('conditions', 12)]], {'help_text': 'Add conditional rules to show or hide fields depending on the value of other fields in the form.', 'required': False}), 14: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('max_length', 2), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Basic field'}), 15: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('max_length', 2), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Email field'}), 16: ('wagtail.blocks.CharBlock', (), {'help_text': 'A unique reference for this html block', 'label': 'Unique reference name', 'max_length': 255, 'required': True}), 17: ('wagtail.blocks.RichTextBlock', (), {'features': ['bold', 'italic', 'h2', 'h3', 'h4', 'h5', 'ol', 'ul', 'link'], 'required': True}), 18: ('wagtail.blocks.StructBlock', [[('label', 16), ('html', 17), ('rules', 13)]], {'label': 'HTML'}), 19: ('wagtail.blocks.TextBlock', (), {'icon': 'extraicons--heading-icon', 'required': True}), 20: ('wagtail.blocks.ListBlock', (19,), {}), 21: ('wagtail.blocks.CharBlock', (), {'max_length': 255, 'required': False}), 22: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('empty_label', 21), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Dropdown field'}), 23: ('wagtail.blocks.BooleanBlock', (), {'help_text': 'Display these items side by side?', 'required': False}), 24: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('default_value', 3), ('help_text', 4), ('display_side_by_side', 23), ('rules', 13)]], {'label': 'Radio field'}), 25: ('wagtail.blocks.TextBlock', (), {'help_text': 'Enter a value that matches a checkbox choice above to have it ticked by default.', 'icon': 'extraicons--heading-icon', 'required': False}), 26: ('wagtail.blocks.ListBlock', (25,), {}), 27: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('default_value', 26), ('help_text', 4), ('display_side_by_side', 23), ('rules', 13)]], {'label': 'Checkboxes field'}), 28: ('wagtail.blocks.BooleanBlock', (), {'default': False, 'help_text': 'Do you want the checkbox label to display? If not you should populate help_text.', 'required': False}), 29: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('display_checkbox_label', 28), ('help_text', 21), ('rules', 13)]], {'label': 'Checkbox field'}), 30: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Multiselect field'}), 31: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Multiline field'}), 32: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('max_length', 2), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'URL field'}), 33: ('wagtail.blocks.CharBlock', (), {'help_text': 'The value to go into the hidden field.', 'label': 'Value', 'max_length': 255, 'required': True}), 34: ('wagtail.blocks.StructBlock', [[('label', 0), ('default_value', 33)]], {'label': 'Hidden field'}), 35: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Number field'})}, default=None, help_text='Add fields to the form', null=True),
        ),
        migrations.AlterField(
            model_name='formpage',
            name='form',
            field=wagtail.fields.StreamField([('singleline', 14), ('email', 15), ('html', 18), ('dropdown', 22), ('radio', 24), ('checkboxes', 27), ('checkbox', 29), ('multiselect', 30), ('multiline', 31), ('url', 32), ('hidden', 34), ('number', 35)], block_lookup={0: ('wagtail.blocks.CharBlock', (), {'help_text': 'The label of the form field.', 'max_length': 255, 'required': True}), 1: ('wagtail.blocks.BooleanBlock', (), {'default': False, 'help_text': 'Tick this box to make this a required field.', 'required': False}), 2: ('wagtail.blocks.IntegerBlock', (), {'help_text': 'Set a maximum length for this field. e.g. 100', 'required': False}), 3: ('wagtail.blocks.CharBlock', (), {'help_text': 'Set a default value for this field.', 'max_length': 255, 'required': False}), 4: ('wagtail.blocks.CharBlock', (), {'help_text': 'Text to assist the user in populating this field.', 'max_length': 255, 'required': False}), 5: ('wagtail.blocks.ChoiceBlock', [], {'choices': [('show', 'Show'), ('hide', 'Hide')], 'help_text': 'What conditional action would you like to perform on this field?'}), 6: ('wagtail.blocks.CharBlock', (), {'help_text': 'The name of a field in the form', 'max_length': 255, 'required': True}), 7: ('wagtail.blocks.ChoiceBlock', [], {'choices': [('is', 'Is equal to'), ('is_not', 'Is not equal to'), ('greater_than', 'Greater than'), ('greater_than_equal', 'Greater than or equal to'), ('less_than', 'Less than'), ('less_than_equal', 'Less than or equal to'), ('contains', 'Contains'), ('starts-with', 'Starts with'), ('ends-with', 'Ends with')]}), 8: ('wagtail.blocks.CharBlock', (), {'help_text': '', 'max_length': 255, 'required': True}), 9: ('wagtail.blocks.StructBlock', [[('field_name', 6), ('rule', 7), ('value', 8)]], {}), 10: ('wagtail.blocks.ChoiceBlock', [], {'choices': [('is_blank', 'Is blank'), ('is_not_blank', 'Is not blank')]}), 11: ('wagtail.blocks.StructBlock', [[('field_name', 6), ('rule', 10)]], {}), 12: ('wagtail.blocks.StreamBlock', [[('condition', 9), ('blank_condition', 11)]], {'required': False}), 13: ('wagtail.blocks.StructBlock', [[('action', 5), ('conditions', 12)]], {'help_text': 'Add conditional rules to show or hide fields depending on the value of other fields in the form.', 'required': False}), 14: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('max_length', 2), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Basic field'}), 15: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('max_length', 2), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Email field'}), 16: ('wagtail.blocks.CharBlock', (), {'help_text': 'A unique reference for this html block', 'label': 'Unique reference name', 'max_length': 255, 'required': True}), 17: ('wagtail.blocks.RichTextBlock', (), {'features': ['bold', 'italic', 'h2', 'h3', 'h4', 'h5', 'ol', 'ul', 'link'], 'required': True}), 18: ('wagtail.blocks.StructBlock', [[('label', 16), ('html', 17), ('rules', 13)]], {'label': 'HTML'}), 19: ('wagtail.blocks.TextBlock', (), {'icon': 'extraicons--heading-icon', 'required': True}), 20: ('wagtail.blocks.ListBlock', (19,), {}), 21: ('wagtail.blocks.CharBlock', (), {'max_length': 255, 'required': False}), 22: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('empty_label', 21), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Dropdown field'}), 23: ('wagtail.blocks.BooleanBlock', (), {'help_text': 'Display these items side by side?', 'required': False}), 24: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('default_value', 3), ('help_text', 4), ('display_side_by_side', 23), ('rules', 13)]], {'label': 'Radio field'}), 25: ('wagtail.blocks.TextBlock', (), {'help_text': 'Enter a value that matches a checkbox choice above to have it ticked by default.', 'icon': 'extraicons--heading-icon', 'required': False}), 26: ('wagtail.blocks.ListBlock', (25,), {}), 27: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('default_value', 26), ('help_text', 4), ('display_side_by_side', 23), ('rules', 13)]], {'label': 'Checkboxes field'}), 28: ('wagtail.blocks.BooleanBlock', (), {'default': False, 'help_text': 'Do you want the checkbox label to display? If not you should populate help_text.', 'required': False}), 29: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('display_checkbox_label', 28), ('help_text', 21), ('rules', 13)]], {'label': 'Checkbox field'}), 30: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('choices', 20), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Multiselect field'}), 31: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Multiline field'}), 32: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('max_length', 2), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'URL field'}), 33: ('wagtail.blocks.CharBlock', (), {'help_text': 'The value to go into the hidden field.', 'label': 'Value', 'max_length': 255, 'required': True}), 34: ('wagtail.blocks.StructBlock', [[('label', 0), ('default_value', 33)]], {'label': 'Hidden field'}), 35: ('wagtail.blocks.StructBlock', [[('label', 0), ('required', 1), ('default_value', 3), ('help_text', 4), ('rules', 13)]], {'label': 'Number field'})}, default=None, help_text='Add fields to the form', null=True),
        ),
    ]



# anyascii is a Wagtail dependency
import re
from anyascii import anyascii
from django.utils.text import slugify
from wagtail.rich_text import expand_db_html


def clean_form_field_name(label):
    # anyascii will return an ascii string while slugify wants a
    # unicode string on the other hand, slugify returns a safe-string
    # which will be converted to a normal str
    return str(slugify(str(anyascii(label))))


def modified_html(value):
    external_link_pattern = r'<a href="(http[s]?://[^"]*)">([^<]*)</a>'
    hyperlink_replacement = r'<a href="\1" rel="noopener noreferrer" target="_blank">\2</a>'
    # Get the source the value
    source = value.source
    # Use expand_db_html to extract html data
    expanded_html = expand_db_html(source)
    # Replace all external links with ones that open in a new tab
    modified_html = re.sub(external_link_pattern, hyperlink_replacement, expanded_html)
    return modified_html


def get_client_ip(request):
    # First check X-Forwarded-For if behind a proxy/load balancer
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        # Could contain multiple IPs, client is the first
        ip = x_forwarded_for.split(',')[0].strip()
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


from django.apps import AppConfig
from django.conf import settings


class HeadlessWafBuilderConfig(AppConfig):
    name = 'headless_waf_builder'
    label = 'headless_waf_builder'
    verbose_name = 'Headless WAF Builder'
    default_auto_field = 'django.db.models.AutoField'

    def ready(self):
        # Handling Django recaptcha required GOOGLE_RECAPTCHA_PUBLIC_KEY & GOOGLE_RECAPTCHA_PRIVATE_KEY
        # as this will be need to set in project django settings
        if not hasattr(settings, 'SILENCED_SYSTEM_CHECKS'):
            setattr(settings, 'SILENCED_SYSTEM_CHECKS', ['django_recaptcha.recaptcha_test_key_error'])
        else:
            if 'django_recaptcha.recaptcha_test_key_error' not in settings.SILENCED_SYSTEM_CHECKS:
                settings.SILENCED_SYSTEM_CHECKS.append('django_recaptcha.recaptcha_test_key_error')




from django.middleware.csrf import get_token
from django.http import JsonResponse
from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.middleware.csrf import CsrfViewMiddleware
from django_recaptcha.client import submit
from ninja import NinjaAPI
from ninja.errors import ValidationError
from headless_waf_builder.models import FormPage, EmailFormPage
from headless_waf_builder.utils import get_client_ip
from .schemas import FormPageSchema, EmailFormPageSchema, FormPageUnion, ThanksPageSchema, FormPostSchema, JSONResponse
from .tasks import send_form_page_email

headless_waf_builder_api = NinjaAPI(csrf=True, docs_url='/docs', title='Headless WAF Builder API')
api = headless_waf_builder_api


@api.exception_handler(ValidationError)
def custom_validation_errors(request, exc):
    return api.create_response(request, {"detail": exc.errors}, status=422)


# Retrieve CSRF Token
@api.get("/csrf/")
def get_csrf_token(request):
    token = get_token(request)
    return JsonResponse({"csrftoken": token})


def validate_csrf(request):
    csrf_middleware = CsrfViewMiddleware(get_response=lambda req: None)
    try:
        csrf_middleware.process_view(request, None, (), {})
        return True
    except PermissionDenied as e:
        print("Error with validating the CSRF token:", e)
        return False


@api.get(
    "/form_by_path/{path:path}",
    response={200: FormPageUnion, 404: JSONResponse, 500: JSONResponse},
    operation_id="get_form_by_path"
)
def form_by_path(request, path):
    try:
        # Search for forms with the given path - combine both form types
        form_pages = list(FormPage.objects.filter(url_path__icontains=path))
        email_form_pages = list(EmailFormPage.objects.filter(url_path__icontains=path))
        all_forms = form_pages + email_form_pages

        # Handle results
        if not all_forms:
            return 404, {"message": "No form found with the specified path"}
        elif len(all_forms) > 1:
            return 500, {"message": f"Found {len(all_forms)} forms with that path. Please use a more specific path."}

        # Get the single form that found
        form_page = all_forms[0]

        # Use the right schema for the right form page
        if isinstance(form_page, EmailFormPage):
            form = EmailFormPageSchema.from_orm(form_page)
        else:
            form = FormPageSchema.from_orm(form_page)
        return 200, form

    except Exception as e:
        print(f"Error retrieving form: {e}")
        return 500, {"message": "Internal server error while getting form"}


@api.post(
    "/form_by_path/",
    response={204: ThanksPageSchema, 403: JSONResponse, 404: JSONResponse, 500: JSONResponse},
    operation_id="post_form_by_path"
)
def form_by_path(request, data: FormPostSchema):
    try:
        # First, validate CSRF token
        if not validate_csrf(request):
            print("CSRF validation failed")
            return 403, {"message": "CSRF validation failed. Please refresh the page and try again."}

        # Checking if reCAPTCHA validation is required
        recaptcha_validation_required = False

        # Search for forms with the given path - try both form types
        form_pages = list(FormPage.objects.filter(url_path__icontains=data.path))
        email_form_pages = list(EmailFormPage.objects.filter(url_path__icontains=data.path))
        all_forms = form_pages + email_form_pages

        # Handle results
        if not all_forms:
            return 404, {"message": "No form found with the specified path"}
        elif len(all_forms) > 1:
            return 500, {"message": f"Found {len(all_forms)} forms with that path. Please use a more specific path."}

        # Get the single form that found
        form_page = all_forms[0]

        # Check if this form requires reCAPTCHA
        if hasattr(form_page, 'use_google_recaptcha') and form_page.use_google_recaptcha:
            recaptcha_validation_required = True

        # Validating reCAPTCHA token if required recaptcha for the form
        if recaptcha_validation_required:
            recaptcha_token = data.recaptcha_token

            # Skip validation if no token is provided but print a warning
            if not recaptcha_token:
                print("WARNING: Form requires reCAPTCHA but no token was provided")
                return 403, {"message": "reCAPTCHA validation required. Please refresh and try again."}

            # Get the client IP address
            client_ip = get_client_ip(request)

            # Validate the token
            recaptcha_response = submit(
                recaptcha_token,
                settings.GOOGLE_RECAPTCHA_PRIVATE_KEY,
                client_ip
            )

            # For reCAPTCHA v2, check the validity
            if not recaptcha_response.is_valid:
                if 'timeout-or-duplicate' in recaptcha_response.error_codes:
                    return 403, {"message": "reCAPTCHA token has expired or duplicated. Please submit the form again later."}
                return 403, {"message": "reCAPTCHA validation failed. Please try again."}

            # For reCAPTCHA v3, checking score
            has_score = hasattr(recaptcha_response, 'extra_data') and 'score' in recaptcha_response.extra_data
            if has_score and recaptcha_response.extra_data.get('score') < settings.RECAPTCHA_REQUIRED_SCORE:
                return 403, {"message": "reCAPTCHA score too low. Please try again later."}

        # Process the form submission
        form_class = form_page.get_form_class()
        form = form_class(data.form_fields)

        # Letting Frontend handling field required logic
        for field in form.fields.values():
            field.required = False

        # Clean the form data
        form.full_clean()

        # Create submission
        form_page.get_submission_class().objects.create(
            form_data=form.cleaned_data,
            page=form_page,
        )

        # Send the email for the EmailFormPage only if there is a list of to_address
        if isinstance(form_page, EmailFormPage):
            email_data = {
                'form_title': form_page.title,
                'subject': form_page.subject,
                'from_address': form_page.from_address,
                'to_address': form_page.to_address,
                'content': form.cleaned_data,
            }
            if email_data.get('to_address'):
                send_form_page_email.delay(email_data)

        return 204, {
            "thanks_page_title": form_page.thanks_page_title,
            "thanks_page_content": form_page.thanks_page_content
        }
    except Exception as e:
        print(f"Error processing form submission: {e}")
        return 500, {"message": "Internal server error while processing form submission"}



import logging

from django.conf import settings

import headless_waf_builder.constants as consts
from pydantic import Field
from headless_waf_builder.models import FormPage, EmailFormPage
from headless_waf_builder.utils import modified_html, clean_form_field_name
from ninja import Schema
from typing import Literal, List, Optional, Union, Dict, Any, Type, Annotated


# Placeholder for conditional ruleset
class FormRuleConditionSchema(Schema):
    field_name: str
    rule: Literal[
        'is',
        'is_not',
        'greater_than',
        'greater_than_equal',
        'less_than',
        'less_than_equal',
        'contains',
        'starts-with',
        'ends-with',
    ]
    value: str


class FormRuleBlankConditionSchema(Schema):
    field_name: str
    rule: Literal[
        'is_blank',
        'is_not_blank'
    ]


class FormRuleSchema(Schema):
    action: Literal["show", "hide"] = consts.FIELD_ACTION_SHOW
    conditions: Optional[
        List[
            Union[
                FormRuleConditionSchema,
                FormRuleBlankConditionSchema
            ]
        ]
    ] = []

    class Config:
        extra = "ignore"


class BaseFieldSchema(Schema):
    id: int
    field_id: Optional[str] = None
    type: str
    name: str
    clean_name: Optional[str] = None
    label: Optional[str] = None
    required: Optional[bool] = False
    help_text: Optional[str] = None
    rules: Optional[FormRuleSchema] = None


class SingleLineSchema(BaseFieldSchema):
    type: Literal["singleline"] = consts.FIELD_TYPE_SINGLE_LINE
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class MultiLineSchema(BaseFieldSchema):
    type: Literal["multiline"] = consts.FIELD_TYPE_MULTI_LINE
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class HTMLSchema(BaseFieldSchema):
    type: Literal["html"] = consts.FIELD_TYPE_HTML
    html: str


class DropdownSchema(BaseFieldSchema):
    type: Literal["dropdown"] = consts.FIELD_TYPE_DROPDOWN
    choices: Optional[List[str]] = []
    empty_label: Optional[str] = None
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class CheckboxSchema(BaseFieldSchema):
    type: Literal["checkbox"] = consts.FIELD_TYPE_CHECKBOX
    display_checkbox_label: Optional[bool] = False
    help_text: Optional[str] = None


class CheckboxesSchema(BaseFieldSchema):
    type: Literal["checkboxes"] = consts.FIELD_TYPE_CHECKBOXES
    choices: Optional[List[str]] = []
    display_side_by_side: Optional[bool] = None
    default_value: Optional[List[str]] = []


class MultiSelectSchema(BaseFieldSchema):
    type: Literal["multiselect"] = consts.FIELD_TYPE_MULTI_SELECT
    choices: Optional[List[str]] = []
    default_value: Optional[str] = None


class RadioSchema(BaseFieldSchema):
    type: Literal["radio"] = consts.FIELD_TYPE_RADIO
    choices: Optional[List[str]] = []
    display_side_by_side: Optional[bool] = None
    default_value: Optional[str] = None


class UrlSchema(BaseFieldSchema):
    type: Literal["url"] = consts.FIELD_TYPE_URL
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class NumberSchema(BaseFieldSchema):
    type: Literal["number"] = consts.FIELD_TYPE_NUMBER
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class EmailSchema(BaseFieldSchema):
    type: Literal["email"] = consts.FIELD_TYPE_EMAIL
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class SimpleDateSchema(BaseFieldSchema):
    type: Literal["simpledate"] = consts.FIELD_TYPE_SIMPLE_DATE
    max_length: Optional[int] = None
    default_value: Optional[str] = None
    minimum_age: Optional[int] = None
    maximum_age: Optional[int] = None


class PhoneSchema(BaseFieldSchema):
    type: Literal["phone"] = consts.FIELD_TYPE_PHONE
    max_length: Optional[int] = None
    default_value: Optional[str] = None


class HiddenSchema(BaseFieldSchema):
    type: Literal["hidden"] = consts.FIELD_TYPE_HIDDEN
    default_value: str  # Required for hidden fields


# Corrected mapping with all field types
FORM_FIELDS_MAPPING = {
    consts.FIELD_TYPE_SINGLE_LINE: SingleLineSchema,
    consts.FIELD_TYPE_MULTI_LINE: MultiLineSchema,
    consts.FIELD_TYPE_HTML: HTMLSchema,
    consts.FIELD_TYPE_DROPDOWN: DropdownSchema,
    consts.FIELD_TYPE_CHECKBOX: CheckboxSchema,
    consts.FIELD_TYPE_CHECKBOXES: CheckboxesSchema,
    consts.FIELD_TYPE_MULTI_SELECT: MultiSelectSchema,
    consts.FIELD_TYPE_RADIO: RadioSchema,
    consts.FIELD_TYPE_URL: UrlSchema,
    consts.FIELD_TYPE_NUMBER: NumberSchema,
    consts.FIELD_TYPE_EMAIL: EmailSchema,
    consts.FIELD_TYPE_SIMPLE_DATE: SimpleDateSchema,
    consts.FIELD_TYPE_PHONE: PhoneSchema,
    consts.FIELD_TYPE_HIDDEN: HiddenSchema
}

# Form fields union
FormFieldUnion = Annotated[
    Union[
        SingleLineSchema,
        MultiLineSchema,
        HTMLSchema,
        DropdownSchema,
        CheckboxSchema,
        CheckboxesSchema,
        MultiSelectSchema,
        RadioSchema,
        UrlSchema,
        NumberSchema,
        EmailSchema,
        SimpleDateSchema,
        PhoneSchema,
        HiddenSchema
    ],
    Field(discriminator='type')
]


class JSONResponse(Schema):
    message: Optional[str] = None
    details: Optional[dict] = {}


class FormPostSchema(Schema):
    form_fields: Dict[str, Any]
    path: str
    recaptcha_token: Optional[str] = None


class FormFieldSubmission(Schema):
    id: int
    name: str
    type: Optional[str]
    value: Any = None
    required: Optional[bool] = False


class ThanksPageSchema(Schema):
    thanks_page_title: Optional[str] = None
    thanks_page_content: str


class BaseFormPageSchema(Schema):
    class Config:
        model = FormPage
        model_fields = "__all__"
        custom_fields = [
            ('fields', List[FormFieldUnion], []),
            ('form_submission_schema', Dict, {}),
        ]
        populate_by_name = True

    id: int
    title: str
    thanks_page_title: Optional[str] = None
    thanks_page_content: str
    submit_button_text: str
    search_description: Optional[str] = None
    seo_title: Optional[str] = None
    show_in_menus: bool = False
    use_browser_validation: bool = False
    fields: List[FormFieldUnion]
    type: str
    use_google_recaptcha: bool = False
    google_recaptcha_public_key: Optional[str] = None

    @staticmethod
    def resolve_google_recaptcha_public_key(obj):
        # This method is called when serialized or deserializing
        # So we need to handle when a dictionary is passed into this as obj
        if isinstance(obj, dict):
            return obj['google_recaptcha_public_key']

        # This method is called when serialized/deserializing/validating
        # So we need to handle when FormPageSchema is passed
        if isinstance(obj, BaseFormPageSchema):
            return obj.google_recaptcha_public_key

        if obj.use_google_recaptcha:
            if hasattr(settings, 'GOOGLE_RECAPTCHA_PUBLIC_KEY'):
                return settings.GOOGLE_RECAPTCHA_PUBLIC_KEY
            logging.warning('GOOGLE_RECAPTCHA_PUBLIC_KEY need to be set in your settings')
            return None
        return None

    @staticmethod
    def resolve_fields(obj):
        # This method is called when serialized or deserializing
        # So we need to handle when a dictionary is passed into this as obj
        if isinstance(obj, dict):
            return obj['fields']

        # This method is called when serialized/deserializing/validating
        # So we need to handle when FormPageSchema is passed
        if isinstance(obj, BaseFormPageSchema):
            return obj.fields

        # This method is called when serialized/deserializing/validating
        # so we need to handle when a Django Model instance is passed in
        form_fields = []
        all_form_fields = list(obj.get_form_fields())

        for idx, block in enumerate(obj.form):
            try:
                block_schema = FORM_FIELDS_MAPPING.get(block.block_type)
                if not block_schema:
                    print(f"No schema found for block type: {block.block_type}")
                    continue
                block_data = {'type': block.block_type, 'id': idx}

                # Extract block data from the value
                for key, value in block.value.items():
                    if isinstance(value, (str, int, float, bool, type(None))):
                        block_data[key] = value

                    elif key == 'rules' and (isinstance(value, dict) or hasattr(value, 'items')):
                        # Create a dictionary for rules
                        rules_data = {'action': value.get('action')}
                        conditions_stream = value.get('conditions')

                        # Process conditions if they exist
                        if conditions_stream:
                            conditions = []
                            for condition_block in conditions_stream:
                                condition_data = {}
                                # Extract condition properties using dictionary access
                                condition_value = condition_block.value
                                if hasattr(condition_value, 'get'):
                                    condition_data['field_name'] = condition_value.get('field_name')
                                    condition_data['rule'] = condition_value.get('rule')
                                    condition_data['value'] = condition_value.get('value')
                                else:
                                    # If condition_value is a real dictionary
                                    condition_data = {k: v for k, v in condition_value.items()}
                                conditions.append(condition_data)

                            rules_data['conditions'] = conditions

                        block_data[key] = rules_data
                    elif key == 'html' and hasattr(value, 'source'):
                        block_data[key] = modified_html(value)
                    elif hasattr(value, 'raw_data') and isinstance(value.raw_data, list):
                        # Handle ListValue objects using their raw_data attribute
                        block_data[key] = value.raw_data
                    elif hasattr(value, '__iter__') and not isinstance(value, (str, dict)):
                        # Try to convert to a list of simple values
                        block_data[key] = [item for item in value]
                    elif isinstance(value, dict):
                        # For dictionaries, keep only serializable values
                        block_data[key] = {
                            k: v for k, v in value.items()
                            if isinstance(v, (str, int, float, bool, list, dict, type(None)))
                        }
                    else:
                        # Try to get a string representation for other types
                        block_data[key] = str(value)

                # Generate unique clean_name using the index
                if idx < len(all_form_fields):
                    form_field = all_form_fields[idx]
                    field_unique_id = form_field.id if form_field.id else idx
                    base_clean_name = clean_form_field_name(form_field.label)
                    unique_clean_name = f"{base_clean_name}-{field_unique_id}"
                    block_data['clean_name'] = unique_clean_name
                    block_data['name'] = base_clean_name

                instance = block_schema(**block_data)
                form_fields.append(instance)
            except Exception as e:
                print(f"Error processing field: {e}")
        return form_fields


class FormPageSchema(BaseFormPageSchema):
    type: Literal["form_page"] = consts.FORM_PAGE

    class Config:
        model = FormPage


class EmailFormPageSchema(BaseFormPageSchema):
    type: Literal["email_form_page"] = consts.EMAIL_FORM_PAGE
    from_address: str
    to_address: str
    subject: str

    class Config:
        model = EmailFormPage


FormPageUnion = Annotated[
    Union[FormPageSchema, EmailFormPageSchema],
    Field(discriminator='type')
]


from wagtail.admin.panels import FieldPanel
from wagtail.models import Page
from django.db import models

from .abstract_advanced_email_form import AbstractAdvancedEmailForm


class EmailFormPage(AbstractAdvancedEmailForm):

    use_google_recaptcha = models.BooleanField(
        default=False,
        null=False,
        blank=False,
        help_text='Please tick this option to enable Google Recaptcha'
    )

    content_panels = Page.content_panels + AbstractAdvancedEmailForm.content_panels + [
        FieldPanel('use_google_recaptcha')
    ]

    settings_panels = Page.settings_panels + AbstractAdvancedEmailForm.settings_panels

    # Override preview for headless usage
    def serve_preview(self, request, mode_name):
        """
        Disable preview for headless forms. Returns a simple message instead of trying to render templates.
        """
        from django.http import HttpResponse
        return HttpResponse(
            "<h1>Preview Disabled</h1>"
            "<p>This is a headless email form - preview functionality is disabled.</p>"
            "<p>Use the REST API endpoints to interact with this form at <a target='_blank' href='/api/docs'>api/docs</a>.</p>",
            content_type="text/html"
        )



from wagtail import blocks

from .base_field_block import BaseFieldBlock


class SimpleDateFieldBlock(BaseFieldBlock):
    # Reason for this block called SimpleDateFieldBlock due to usage of CharBlock for default_value instead of Date select widget

    choices = None

    html = None

    empty_label = None

    default_value = blocks.CharBlock(
        max_length=10,
        required=False,
        help_text='dd/mm/yyyy e.g. 19/08/1981'
    )

    buttons_style = None

    display_checkbox_label = None

    display_side_by_side = None

    class Meta:
        form_classname = 'waf--field'
        icon = 'extraicons--date'



# Generated by Django 4.2.20 on 2025-05-15 10:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('headless_waf_builder', '0006_added_phone_and_simple_date_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailformfield',
            name='maximum_age',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='emailformfield',
            name='minimum_age',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='formfield',
            name='maximum_age',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='formfield',
            name='minimum_age',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
    ]



from celery import shared_task
from celery.utils.log import get_task_logger
from django.template.loader import render_to_string
from django.utils.html import strip_tags
from django.core.mail import EmailMultiAlternatives
from django.conf import settings

logger = get_task_logger(__name__)


@shared_task(bind=True, max_retries=2, ignore_result=True)
def send_form_page_email(self, email_data):
    try:
        form_title = email_data.get('form_title', '')
        subject = email_data.get('subject', 'Form Submission')
        from_address = email_data.get('from_address', settings.DEFAULT_FROM_EMAIL)
        to_address = email_data.get('to_address', '')
        content = email_data.get('content', {})

        # Processing email template
        form_fields = []
        for key, value in content.items():
            # Skip HTML fields
            if key == 'html-field':
                continue

            # Skip empty values: None, '', [], or {}
            if value in (None, '', [], {}):
                continue

            # Format field name from slug to title
            field_label = key.replace('-', ' ').title()

            # If it's a list, join the string together
            if isinstance(value, list):
                formatted_value = ', '.join(str(v) for v in value if v)
                if not formatted_value:
                    continue  # Skip if list was empty
            else:
                formatted_value = str(value)

            form_fields.append((field_label, formatted_value))

        # Prepare email context data
        email_context = {
            'form_title': form_title,
            'form_fields': form_fields,
        }

        html_content = render_to_string('emails/form_submission.html', email_context)
        plain_content = strip_tags(html_content)

        # Format recipient list of address emails
        recipient_list = [email.strip() for email in to_address.split(',') if email.strip()]

        # Send the email
        email = EmailMultiAlternatives(
            subject=subject,
            body=plain_content,
            from_email=from_address,
            to=recipient_list,
        )
        email.attach_alternative(html_content, "text/html")
        email.send()

        logger.info(f"Email sent successfully to {to_address}")
        return True

    except Exception as e:
        logger.error(f"Failed to send email: {str(e)}")
        self.retry(exc=e, countdown=60)  # Retry after 60 seconds
        return None
